from prijsoog.supermarkets.ah import AhWatcher
from prijsoog.supermarkets.jumbo import JumboWatcher

__author__ = """Pepijn Boers"""
__email__ = "pepijnbb@gmail.com"
__version__ = "__version__ = '0.1.7'"
__all__ = ["AhWatcher", "JumboWatcher"]



#!/usr/bin/env python

"""Tests for `prijsoog` package."""


import re
from prijsoog import AhWatcher, JumboWatcher


def test_ah_watcher():
    """Test Albert Heijn price watcher"""
    AH = AhWatcher()
    products = AH.search("kiwi")

    # Test if there are no html tags in product name
    prod_name = list(products.keys())[0]
    tag_expr = re.compile(r"<.*?>")
    stripped_prod_name = tag_expr.sub("", prod_name)
    assert prod_name == stripped_prod_name

    # Test if product price has the correct format
    prod_price = list(products.values())[0]
    str_prod_price = str(prod_price)
    assert len(str_prod_price.split(".")) == 2


def test_jumbo_watcher():
    """Test Jumbo price watcher"""
    Jumbo = JumboWatcher()
    products = Jumbo.search("kiwi")

    # Test if there are no html tags in product name
    prod_name = list(products.keys())[0]
    tag_expr = re.compile(r"<.*?>")
    stripped_prod_name = tag_expr.sub("", prod_name)
    assert prod_name == stripped_prod_name

    # Test if product price has the correct format
    prod_price = list(products.values())[0]
    str_prod_price = str(prod_price)
    assert len(str_prod_price.split(".")) == 2



"""Top-level package for Prijsoog."""

import requests
from bs4 import BeautifulSoup
from datetime import datetime


CUSTOM_HEADER = {
    "User-Agent": (
        "Mozilla/5.0 (iPhone; CPU iPhone OS 13_2_3 like Maci"
        " OS X) AppleWebKit/605.1.15 (KHTML, like Gecko)"
        " Version/13.0.3 Mobile/15E148 Safari/604.1"
    ),
    "Accept": (
        "text/html,application/xhtml+xml,application/xml;q=0.9,"
        "image/avif,image/webp,image/apng,*/*;q=0.8,application"
        "/signed-exchange;v=b3;q=0.9"
    ),
}


class PriceWatcher:
    """
    Main class that enables product & price retrieval of Dutch supermarkets
    """

    def __init__(self, min_delay=5):
        """Class constructor

        :param min_delay: minimum request delay in sec
        :type min_delay: int
        """
        self.last_request = datetime(2000, 1, 1, 0, 0)
        self.min_delay = min_delay
        self.session = requests.Session()
        self.session.headers.update(CUSTOM_HEADER)

    def search(self, query: str) -> dict:
        """Search different supermarkets for product query

        :param query: product name, e.g. 'appel'
        :type query: str
        :return: results of your search
        :rtype: dict
        """
        pass

    def make_request(self, url):
        """Access website and parse data"""
        response = self.session.get(url)
        data = response.text
        soup = BeautifulSoup(data, "html.parser")
        return soup



from prijsoog._base import PriceWatcher
from datetime import datetime
import time


class JumboWatcher(PriceWatcher):
    """Jumbo price watcher."""

    def search(self, query: str):
        """Search different supermarkets for product query

        :param query: product name, e.g. 'appel'
        :type query: str
        :return: results of your search
        :rtype: dict
        """
        # Make sure there is at least 5 sec interval between requests.
        current_interval = datetime.now() - self.last_request
        if current_interval.seconds < self.min_delay:
            time.sleep(self.min_delay - current_interval.seconds)

        url = f"https://www.jumbo.com/zoeken?searchTerms={query}"
        soup = super().make_request(url)

        # Css selector
        name_css = ".title-link"
        price_css = ".fractional , .whole"

        # Make selection
        selection = {
            "name": [name.contents[0] for name in soup.select(name_css)],
            "price": self._price_filter(soup.select(price_css)),
        }

        return dict(zip(*selection.values()))

    def _price_filter(self, prices):
        """Combine Jumbo price elements '2', ',', '99' into '2.99'"""
        return [price.text for price in prices if len(price.text) > 0]
